from earthdiagnostics.models import Grid
from earthdiagnostics import Utils
from nco import NCOException
from autosubmit.config.log import Log


class Convection(object):
    @staticmethod
    def main_sites(input_file, input_grid, output_file):
        """
        Compute the intensity of convection in the four main convection sites

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing somxl010
        :param input_grid: input grid
        :param output_file: output file name (=> index)
        :return:
        """
        if input_grid in [Grid.ECEARTH_2_3_O1L42, Grid.ECEARTH_3_0_O1L46,
                          Grid.NEMO_3_2_O1L42, Grid.NEMO_3_3_O1L46,
                          Grid.NEMOVAR_O1L42]:
            labrador = [255, 245, 215, 255]
            irminger = [245, 290, 215, 245]
            gin = [260, 310, 245, 291]
            wedell = [225, 280, 1, 50]

        elif input_grid in [Grid.ECEARTH_3_0_O25L46, Grid.ECEARTH_3_0_O25L75,
                            Grid.GLORYS2_V1_O25L75]:
            raise Exception("Option convection not available yet for {0}".format(input_grid))
        else:
            raise Exception("Input grid {0} not recognized".format(input_grid))

        labrador_file = Utils.get_temp_file()
        Convection.site(input_file, labrador, labrador_file)

        irminger_file = Utils.get_temp_file()
        Convection.site(input_file, irminger, irminger_file)

        gin_file = Utils.get_temp_file()
        Convection.site(input_file, gin, gin_file)

        wedell_file = Utils.get_temp_file()
        Convection.site(input_file, wedell, wedell_file)

        nco = Utils.nco
        Utils.rename_variable(labrador_file, 'somxl010', 'Labrador')
        Utils.rename_variable(irminger_file, 'somxl010', 'Irminger')
        Utils.rename_variable(gin_file, 'somxl010', 'GIN')
        Utils.rename_variable(wedell_file, 'somxl010', 'Wedell')

        nco.ncks(input=labrador_file, output=output_file, options='-O -v Labrador')
        nco.ncks(input=irminger_file, output=output_file, options='-A -v Irminger')
        nco.ncks(input=gin_file, output=output_file, options='-A -v GIN')
        nco.ncks(input=wedell_file, output=output_file, options='-A -v Wedell')

    @staticmethod
    def site(input_file, site, output_file):
        """
        Compute the intensity of convection at one site

        Created in October 2013         Author : vguemas@ic3.cat

        :param input_file: input oce file name containing somxl010
        :param site: site to calculate convection on
        :param output_file: output file name (=> index)
        :return:
        """

        Utils.cdo.fldmax(input='-selindexbox,{0[0]},{0[1]},{0[2]},{0[3]} {1}'.format(site, input_file),
                         output=output_file)


def main():
    try:
        Convection.main_sites("/scratch/Earth/jvegas/compsstest/ORCA1_MM_19601101_19610228_grid_T.nc",
                              Grid.ECEARTH_2_3_O1L42, "outconvection.nc")
        Utils.clean()
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)

if __name__ == "__main__":
    main()
