from earthdiagnostics import Utils
from autosubmit.config.log import Log
import os
import shutil
import traceback
from nco import NCOException


class Heat:

    @staticmethod
    def layer(input_file, output_file, upper, lower):
        cdo = Utils.cdo
        nco = Utils.nco

        temp = Utils.get_temp_file()
        temp2 = Utils.get_temp_file()
        heatc_sl_out = Utils.get_temp_file()
        heatc_sl_top = Utils.get_temp_file()
        heatc_sl_bottom = Utils.get_temp_file()
        heatc_sl_invert = Utils.get_temp_file()
        e3tfile = Utils.get_temp_file()

        # Use this names for debugging
        # temp = 'temp.nc'
        # temp2 = 'temp2.nc'
        # heatc_sl_out = 'heatc_sl_out.nc'
        # heatc_sl_top = 'heatc_sl_top.nc'
        # heatc_sl_bottom = 'heatc_sl_bottom.nc'
        # heatc_sl_invert = 'heatc_sl_invert.nc'
        # e3tfile = 'e3t_file.nc'

        nco.ncap2(input='mesh_zgr.nc', output=e3tfile, options='-v -O -s "heatc_sl=tmask*e3t"'),
        nco.ncrename(input=e3tfile, output=e3tfile, options='-d t,time -d z,deptht')

        cdo.seltimestep('1', input=input_file, output=temp)

        nco.ncks(input=temp, output=temp, options='-O -v votemper')
        nco.ncrename(input=temp, output=temp, options='-v votemper,heatc_sl')
        cdo.mul(input=' '.join([temp, e3tfile]), output=heatc_sl_out)
        os.remove(e3tfile)
        nco.ncks(input=heatc_sl_out, output=temp, options='-O -d deptht,{0}.0,{1}.0'.format(upper, lower))

        # perform the integration of ohc down to that level (main contribution)
        nco.ncap2(input=temp, output=heatc_sl_top, options='-O -s "heatc_sl=heatc_sl.total($deptht)"')

        # now extract a few levels below, to compute the residual ohc
        # obtain the weight for the extra level containing the 300 m
        # deptht in the gridT files is positive
        # weight = (300.0 - depth_top)/(depth_bottom - depth_top)
        # and add the thickness down to 300 m in the next layer
        nco.ncpdq(input=heatc_sl_out, output=heatc_sl_invert, options='-a "-deptht"')
        nco.ncks(input=heatc_sl_invert, output=heatc_sl_invert, options='-O -d deptht,0,0,1')
        nco.ncrename(input=heatc_sl_invert, output=heatc_sl_invert, options='-v deptht,layerthcknss')

        nco.ncks(input=heatc_sl_out, output=heatc_sl_bottom,
                 options='-O -d deptht,{0}.0,{1}.0'.format(lower, lower + 200))
        nco.ncks(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -d deptht,0,0,1')
        nco.ncrename(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-v deptht,layerthcknss')

        os.remove(heatc_sl_out)

        # Here, add the residual contribution, before adding it to the main contribution
        nco.ncbo(input=' '.join([heatc_sl_bottom, heatc_sl_invert]), output=temp,
                 options='-A --op_typ=sub -v layerthcknss')

        nco.ncrename(input=temp, output=temp, options='-v layerthcknss,heatc_sl')
        nco.ncap2(input=heatc_sl_invert, output=temp2, options='-s "heatc_sl=($3 - layerthcknss)"')
        os.remove(heatc_sl_invert)
        nco.ncbo(input=' '.join([temp, temp2]), output=temp,  options='--op_typ=/ -v heatc_sl')
        os.remove(temp2)
        nco.ncrename(input=temp, output=temp, options='-v heatc_sl,factor')
        nco.ncks(input=temp, output=heatc_sl_bottom, options='-A -v factor')
        nco.ncap2(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -s "heatc_sl=(factor * heatc_sl)"')
        nco.ncwa(input=heatc_sl_bottom, output=heatc_sl_bottom, options='-O -a deptht')
        nco.ncbo(input=' '.join([heatc_sl_top, heatc_sl_bottom]), output=temp,  options='--op_typ=+ -v heatc_sl')
        os.remove(heatc_sl_top)
        os.remove(heatc_sl_bottom)
        nco.ncap2(input=temp, output=temp, options='-s "heatc_sl=1020.0*4000*heatc_sl"')

        Utils.setminmax(temp, ['heatc_sl'])

        shutil.move(temp, output_file)
        os.remove(temp)


def main():
    try:
        Heat.layer("ORCA1_MM_19601101_19610228_grid_T.nc", "out.nc", 300, 2000)
    except NCOException as ex:
        Log.error('Exception {0}: stderr: {1}, stdout: {2}',  ex.msg, ex.stderr, ex.stdout)
        Log.error(traceback.format_exc())


if __name__ == "__main__":
    main()
